from enum import Enum


class Paths(Enum):
    ATTRIBUTES_FOR_FACETING = "attributes-for-faceting"
    DISPLAYED_ATTRIBUTES = "displayed-attributes"
    DISTINCT_ATTRIBUTE = "distinct-attribute"
    DOCUMENTS = "documents"
    DUMPS = "dumps"
    HEALTH = "health"
    INDEXES = "indexes"
    KEYS = "keys"
    SEARCH = "search"
    SEARCHABLE_ATTRIBUTES = "searchable-attributes"
    SETTINGS = "settings"
    STATS = "stats"
    STOP_WORDS = "stop-words"
    SYNONYMS = "synonyms"
    RANKING_RULES = "ranking-rules"
    UPDATES = "updates"
    VERSION = "version"
